#include <boost/mpi/environment.hpp>
#include <boost/mpi/communicator.hpp>
#include <boost/mpi.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/string.hpp>
#include <iostream>
#include <vector>
#include <sstream>
#include <boost/array.hpp>
#include <boost/mpi/collectives.hpp>

namespace mpi = boost::mpi;
using namespace std;

class Stat
{
	public:
		friend class boost::serialization::access;
		template<class Archive>
		void serialize(Archive & ar, const unsigned int version)
		{
			ar & id;
			ar & br_poena;
			ar & br_skokova;
			ar & br_asistencija;
		}
		
	public:
		int id;
		int br_poena;
		int br_skokova;
		int br_asistencija;
		
		Stat(){}
		Stat(int i, int brp, int brs, int bra)
		{
			id=i;
			br_poena=brp;
			br_skokova=brs;
			br_asistencija=bra;
		}
};

class Kosarkas
{
	public:
		friend class boost::serialization::access;
		template<class Archive>
		void serialize(Archive & ar, const unsigned int version)
		{
			ar & id;
			ar & ime;
			ar & br_poena;
			ar & br_skokova;
			ar & br_utakmica;
			ar & br_asistencija;
		}
		
	public:
		int id;
		char ime[20];
		int br_poena;
		int br_skokova;
		int br_utakmica;
		int br_asistencija;
	
		Kosarkas(){}
		Kosarkas(int i, char *im)
		{
			id=i;
			strcpy(ime,im);
		}
			
		void AddStat(Stat s)
		{
			this->br_poena+=s.br_poena;
			this->br_skokova+=s.br_skokova;
			this->br_asistencija=br_asistencija;
		}
	
		bool IsPlayer(int id)
		{
			if(this->id==id)
				return true;
			return false;
		}
		
		
		//Kosarkas::operation>(Kosarkas k)
	//	{
	//		if(mvp_rate(this)>mvp_rate(k));
	//			return this;
	//		return k;
	//	}
};

float mvp_rate(Kosarkas k)
{
	float mvp;
	int brojilac=0;
		
	brojilac = k.br_poena*2+k.br_asistencija*3+k.br_skokova*3;
	mvp=brojilac/k.br_utakmica;
	
	return mvp;
}

int main(int argc, char* argv[]) 
{
	mpi::environment env(argc, argv);
	mpi::communicator world;
	
	int br_kos;
	int br_stat;
	vector<Kosarkas> kosarkasi;
	vector<Stat> statistike;
	
	 if(world.rank() == 0) 
	{
		int id;
		char ime[20];
		
		int br_poena, br_skokova,br_asistencija;
			
		FILE *fk=fopen("kosarkasi.txt","r");
		fscanf(fk,"%d",&br_kos);
		for(int i=0;i<br_kos;i++)
		{
			fscanf(fk,"%d %s", &id, ime);
			Kosarkas kos(id,ime);
			kosarkasi.push_back(kos);
		}
		fclose(fk);
		
		FILE *fs=fopen("statistika.txt","r");
		fscanf(fs,"%d",&br_stat);
		for(int i=0;i<br_stat;i++)
		{
			fscanf(fs,"%d %d %d %d", &id, &br_poena, &br_skokova, &br_asistencija);
			Stat st(id,br_poena,br_skokova,br_asistencija);
			statistike.push_back(st);
		}
		fclose(fs);
	}
	
	broadcast(world,br_kos,0);
	broadcast(world,br_stat,0);
	
	int podela=br_kos/world.size();
	Kosarkas *mojikosarkasi=new Kosarkas[podela];
	
	broadcast(world,statistike,0);
	scatter(world,kosarkasi,mojikosarkasi,podela,0);
	
	for(int i=0;i<podela;i++)
	{
		for(int j=0;j<br_stat;j++)
			if(mojikosarkasi[i].IsPlayer(statistike[j].id))
				mojikosarkasi[i].AddStat(statistike[j]);
	}
	
	Kosarkas naj_mvp();
	naj_mvp=mojikosarkasi[0];
	//float max_mvp = mvp_rate(mojikosarkasi[0]);
	printf("prosao %d: naj_mvp %s\n",world.rank(),naj_mvp.ime);
	for(int i=1;i<podela;i++)
		{
			if(mojikosarkasi[i]>naj_mvp)//(mvp_rate(mojikosarkasi[i])>max_mvp)
			{
				//max_mvp=mvp_rate(mojikosarkasi[i]);
				naj_mvp=mojikosarkasi[i];
			}
		}
		
	for(int i=0;i<world.size();i++)
		printf("%d, nasao  mvp: %s\n",world.rank(),naj_mvp.ime);
	Kosarkas mvp;
	reduce(world,mvp,naj_mvp,mpi::maximum<Kosarkas>(),0);
	
	if(world.rank()==0)
	{
		printf("MVP: %s\n",mvp.ime);
		}
}
