#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <time.h>

#define COMM MPI_COMM_WORLD

#define N 10
#define M 10
#define K 5

typedef struct{
	int x, y, item, win, loss;
} player;

int calcRowsSpan(int np, int id, int numRows, int *firstRow, int *lastRow)
{
	int piece = numRows / np;
	*firstRow = id*piece;
	*lastRow = id*piece + piece - 1;
}

int getItem()
{
	return ((double) rand()/RAND_MAX)*3;
}

void getNeighbours(int x, int y, int *count, int *xR, int *yR)
{
	int xN[8], yN[8], i;
	
	int counter = 0;
	
	xN[0]=x-1;	xN[1]=x-1;	xN[2]=x-1;	xN[3]=x;
	xN[4]=x;	xN[5]=x+1;	xN[6]=x+1;	xN[7]=x+1;
	
	yN[0]=y-1;	yN[1]=y;	yN[2]=y+1;	yN[3]=y-1;
	yN[4]=y+1;	yN[5]=y-1;	yN[6]=y;	yN[7]=y+1;
	
	for (i = 0; i < 8; i++)
	{
		if (xN[i]<N && xN[i]>=0 && yN[i]<M && yN[i]>=0)
		{
			xR[counter] = xN[i];
			xR[counter] = yN[i];
			counter++;
		}
	}
	*count = counter;
}

void fillPlayers(player *ps, int firstRow, int lastRow, int flag)
{
	int i,j,counter = 0;
	
	for (i = firstRow; i <= lastRow; i++)
	{
		for (j = 0; j < N; j++)
		{
			if (!flag)
			{
				ps[counter].x=j;
				ps[counter].y=i;
			}
			ps[counter].item=getItem();
			counter++;
		}
	}
}

int main(int argc, char* argv[])
{
	int id, np, i, j, z, q, itemN, result, indexN, maxWin = 0;
	
	MPI_Init(&argc,&argv);
	MPI_Comm_size(COMM,&np);
	MPI_Comm_rank(COMM,&id);
	MPI_Status status;
	
	srand(time(NULL)+id+99);
	
	int numBlocks[1];
	MPI_Datatype myStruct, types[1];
	
	MPI_Aint offset[1], extentInt;
	MPI_Type_extent(MPI_INT,&extentInt);
	
	numBlocks[0]=5;
	offset[0]=0;
	types[0]=MPI_INT;
	MPI_Type_struct(1,numBlocks, offset, types, &myStruct);
	MPI_Type_commit(&myStruct);
	
	int pkm[3][3];
	
	pkm[0][0]=-1;	pkm[1][0]=0;	pkm[2][0]=1;
	pkm[0][1]=1;	pkm[1][1]=-1;	pkm[2][1]=0;
	pkm[0][2]=0;	pkm[1][2]=1;	pkm[2][2]=-1;
	
	int firstRow, lastRow;
	calcRowsSpan(np,id,M,&firstRow,&lastRow);
	
	int totalPlayersPerProcess = M/np * N;
	player players[totalPlayersPerProcess];
	player neighArrTop[N];
	player neighArrBot[N];
	
	fillPlayers(players,firstRow,lastRow,0);
	
	int xN[8],yN[8],countN = 0;
	int resultsMatrix[K][3], resultsMatrixFinal[K][3], resultArr[3], rA[3];
	
	for (i = 0; i < K; i++)
	{
		for (j = 0; j < 3; j++)
		{
			resultsMatrix[i][j]=0;
		}
	}
	
	for (i = 0; i < K; i++)
	{
		fillPlayers(players, firstRow, lastRow, 1);
		
		//top rows send
		if (id!=0)
		{
			MPI_Send(&players[0],N,myStruct,id-1,11,COMM);
			MPI_Recv(neighArrTop,N,myStruct,id-1,22,COMM,&status);
		}
		//bottom rows send
		if (id<np-1)
		{
			MPI_Send(&players[np/N*M],N,myStruct,id+1,22,COMM);
			MPI_Recv(neighArrBot,N,myStruct,id+1,11,COMM,&status);
		}
		
		for (j = 0; j < totalPlayersPerProcess; j++)
		{
			getNeighbours(players[j].x,players[j].y,&countN,xN,yN);
			
			for (z = 0; z < countN; z++)
			{
				indexN = yN[z]*N + xN[z];
				itemN = players[j].item;
				
				if (indexN < totalPlayersPerProcess)
				{
					itemN = players[indexN].item;
				}
				
				result = pkm[players[j].item][itemN];
				
				switch (result)
				{
					case 1:
						players[j].win++;
						resultsMatrix[i][players[j].item]++;
						break;
					case 0:
						players[j].loss++;
						break;
				}
				
				for (q = 0; q < N; q++)
				{
					if (xN[z] == neighArrBot[q].x && yN[z] == neighArrBot[q].y)
					{
						itemN = neighArrBot[q].item;
						break;
					}
				}
				
				result = pkm[players[j].item][itemN];
				
				switch (result)
				{
					case 1:
						players[j].win++;
						resultsMatrix[i][players[j].item]++;
						break;
					case 0:
						players[j].loss++;
						break;
				}
				
				for (q = 0; q < N; q++)
				{
					if(xN[z] == neighArrTop[q].x && yN[z] == neighArrTop[q].y)
					{
						itemN = neighArrBot[q].item;
						break;
					}
				}
				result = pkm[players[j].item][itemN];
				
				switch (result)
				{
					case 1:
						players[j].win++;
						resultsMatrix[i][players[j].item]++;
						break;
					case 0:
						players[j].loss++;
						break;
				}
			}
			if (players[j].win > maxWin)
			{
				maxWin = players[j].win;
			}
			
		}
		
		MPI_Reduce(&resultsMatrix[i][0],resultArr,3,MPI_INT,MPI_SUM,0,COMM);
		
		if (id == 0)
		{
			resultsMatrixFinal[i][0] = resultArr[0];
			resultsMatrixFinal[i][1] = resultArr[1];
			resultsMatrixFinal[i][2] = resultArr[2];
		}
	}
	
	rA[0]=0;rA[1]=0;rA[2]=0;
	if (id == 0)
	{
		printf("K \t papir \t kamen \t makaze\n");
		for (i = 0; i < K; i++)
		{
			rA[0] += resultsMatrixFinal[i][0];
			rA[1] += resultsMatrixFinal[i][1];
			rA[2] += resultsMatrixFinal[i][2];
			
			printf("%d \t %d \t %d \t %d",i+1,rA[0],rA[1],rA[2]);
		}
		
		printf("\nNajbolji takmicari:\n");
	}
	
	
	int maxTotal;
	MPI_Reduce(&maxWin, &maxTotal, 1, MPI_INT, MPI_MAX, 0, COMM);
	MPI_Bcast(&maxTotal, 1, MPI_INT, 0, COMM);
	
	for (i = 0; i < totalPlayersPerProcess; i++)
	{
		if (players[i].win = maxWin)
			{
				printf("x:%d\ty:%d\n",players[i].x,players[i].y);
			}
		
	}
	
	MPI_Finalize();
	return 0;
}
