#include <boost/mpi/environment.hpp>
#include <boost/mpi/communicator.hpp>
#include <boost/mpi.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/string.hpp>
#include <iostream>
#include <vector>
#include <sstream>
#include <boost/array.hpp>
#include <boost/mpi/collectives.hpp>

#define BROJ_STATISTIKA 10

namespace mpi = boost::mpi;
using namespace std;

class Stat
{
public:
	int id, br_poena, br_skokova, br_asistencija;
	Stat(){}
	Stat(int id, int br_poena, int br_skokova, int br_asistencija)
	{
		this->id = id;
		this->br_poena = br_poena;
		this->br_skokova = br_skokova;
		this->br_asistencija = br_asistencija;
	}
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & id;
		ar & br_poena;
		ar & br_skokova;
		ar & br_asistencija;
	}
};

class Kosarkas
{
public:
	int id, br_poena, br_asistencija, br_skokova, br_utakmica;
	char ime[50];
	
	friend class boost::serialization::access;
	template<class Archive>
	void serialize(Archive & ar, const unsigned int version)
	{
		ar & id;
		ar & br_poena;
		ar & br_asistencija;
		ar & br_skokova;
		ar & br_utakmica;
		ar & ime;
	}
	
	Kosarkas()
	{
		br_poena = 0;
		br_asistencija = 0;
		br_skokova = 0;
		br_utakmica = 0;
	}
	Kosarkas(int id, string ime)
	{
		this->id = id;
		strcpy(this->ime, ime.c_str());
		br_poena = 0;
		br_asistencija = 0;
		br_skokova = 0;
		br_utakmica = 0;
	}
	
	void AddStat(Stat stat)
	{
		if (IsPlayer(stat.id))
		{
			cout << "Broj poena " << br_poena << " za id " << stat.id << endl;
			br_poena += stat.br_poena;
			br_asistencija += stat.br_asistencija;
			br_skokova += stat.br_skokova;
			br_utakmica++;
		}
	}
	
	bool IsPlayer(int id)
	{
		return (this->id == id);
	}
	
	double Mvp()
	{
		return 1.0 * (br_poena * 2 + br_asistencija * 3 + br_skokova * 3) / br_utakmica;
	}
	
	bool operator> (Kosarkas drugi)
	{
		return Mvp() > drugi.Mvp();
	}
	
	/*
	 * nece ovo
	ostream &operator << ()
	{
		ostream << "ime: " << this->ime << endl;
		ostream << "broj poena: " << br_poena << endl;
		ostream << "broj asistencija: " << br_asistencija << endl;
		ostream << "broj skokova: " << br_skokova << endl;
		ostream << "broj utakmica: " << br_utakmica << endl;
		return ostream;
	}
	*/
	
};

int main (int argc, char **argv)
{
	int brKosarkasa, brStat, nbrKosarkasa, nbrStat;
	char ime[50];
	int id, br_poena, br_asistencija, br_skokova;
	Kosarkas *kosarkasi, *mojiKosarkasi, *mvpKosarkasi;
	Kosarkas mvp;
	Stat *statistika = new Stat[BROJ_STATISTIKA];
	FILE *f;
	
	mpi::environment env(argc, argv);
	mpi::communicator world;
	
	if (world.rank() == 0)
	{
		f = fopen("podaci.dat", "r");
		fscanf(f, "%d", &brKosarkasa);
	}
	nbrKosarkasa = brKosarkasa / world.size();
	broadcast(world, nbrKosarkasa, 0);
	mojiKosarkasi = new Kosarkas[nbrKosarkasa];
	
	if (world.rank() == 0)
	{
		kosarkasi = new Kosarkas[brKosarkasa];
		for (int i = 0; i < brKosarkasa; i++)
		{
			fscanf(f, "%d%s", &id, ime);
			Kosarkas k;
			k.id = id;
			strcpy(k.ime, ime);
			kosarkasi[i] = k;
		}
		fscanf(f, "%d", &brStat);
	}
	scatter(world, kosarkasi, mojiKosarkasi, nbrKosarkasa, 0);
	broadcast(world, brStat, 0);
	nbrStat = 0;
	while (nbrStat < brStat)
	{
		int opseg;
		if ((brStat - nbrStat) > BROJ_STATISTIKA)
			opseg = BROJ_STATISTIKA;
		else
			opseg = brStat - nbrStat;
		cout << "Opseg " << opseg << endl;
		cout << "brSTat " << brStat << endl;
		if (world.rank() == 0)
		{
			for (int i = nbrStat; i < (nbrStat + opseg); i++)
			{
				fscanf(f, "%d%d%d%d", &id, &br_poena, &br_asistencija, &br_skokova);
				Stat s(id, br_poena, br_skokova, br_asistencija);
				statistika[i] = s;
			}
		}
		broadcast(world, statistika, BROJ_STATISTIKA, 0);
		
		for (int i = 0; i < opseg; i++)
			for (int j = 0; j < nbrKosarkasa; j++)
				mojiKosarkasi[j].AddStat(statistika[i]);
		nbrStat += opseg;
	}
		
	mvpKosarkasi = new Kosarkas[world.size()];
	
	mvp = mojiKosarkasi[0];
	for (int i = 1; i < nbrKosarkasa; i++)
	{
		if (mojiKosarkasi[i] > mvp)
			mvp = mojiKosarkasi[i];
	}
	gather(world, mvp, mvpKosarkasi, 0);
	if (world.rank() == 0)
	{
		fclose(f);
		for (int i = 1; i < world.size(); i++)
			if (mvpKosarkasi[i] > mvp)
				mvp = mvpKosarkasi[i];
		cout << "Mvp takmicenja je" << endl;
		cout << "ime: " << mvp.ime << endl;
		cout << "broj poena: " << mvp.br_poena << endl;
		cout << "broj asistencija: " << mvp.br_asistencija << endl;
		cout << "broj skokova: " << mvp.br_skokova << endl;
		cout << "broj utakmica: " << mvp.br_utakmica << endl;
	}
	
	return 0;
}
