#include <boost/mpi/environment.hpp>
#include <boost/mpi/communicator.hpp>
#include <boost/mpi.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/access.hpp>
#include <boost/serialization/string.hpp>
#include <iostream>
#include <vector>
#include <sstream>
#include <boost/array.hpp>
#include <boost/mpi/collectives.hpp>

namespace mpi = boost::mpi;
using namespace std;

class Stat{
	public:
		int id;
		int br_poena;
		int br_skokova;
		int br_asistencija;
	public:
		Stat(){};
		~Stat(){};
		Stat(int a, int b, int, c, int d){
			this->id = a;
			this->br_poena = b;
			this->br_skokova = c;
			this->br_asistencija = d;
		}
		friend class boost::serialization::access;
		template <class Archive>;
		void serialization(Archive & ar, unsigned int version){
			ar & id;
			ar & br_poena;
			ar & br_skokova;
			ar & br_asistencija;
		}
}

class Kosarkas{
	public:
		int id, 
		int ime;
		int br_poena;
		int br_asistencija;
		int br_skokova;
		int br_utakmica;
	public:
		Kosarkas(){};
		~Kosarkas(){};
		Kosarkas(int a, int b){
			this->id = a;
			this->ime = b;
			this->br_poena = 0;
			this->br_asistencija = 0;
			this->br_skokova = 0;
			this->br_utakmica = 0;
		}
		
		void AddStat(int a, int b, int c){
			this->br_poena += a;
			this->br_asistencija += b;
			this->br_skokova += c;
		}
		
		bool IsPlayer(int a){
			if (this->id == a) 
				return true;
			else
				return false;
		}
		
		double IndeksniPoeni(){
			double poeni = (this->br_poena * 2 +
							this->br_asistencija * 3 +
							this->br_skokova * 3) / this->br_utakmica;
			return poeni;
		}
		
		bool operator > (Kosarkas &k){
			
			if (this->IndeksniPoeni() > k.IndeksniPoeni())
				return true;
			else
				return false;
		}
		
		friend class boost::serialization::access;
		template <class Archive>;
		void serialization(Archive & ar, unsigned int version){
			ar & id;
			ar & ime;
			ar & br_poena;
			ar & br_asistencija;
			ar & br_skokova;
			ar & br_utakmica;
		}
}

BOOST_IS_MPI_DATATYPE(Kosarkas);

int main(int argc, char *argv[]){
	
	mpi::environment env(argc, argv);
	mpi::communicator world;
	
	vector <Kosarkas> kosarkasi;
	Kosarkas *mojiKosarkasi;
	
	int poKoliko, broj_kosarkasa, i;
	
	/* ucitavanje podataka */
	if(world.rank() == 0) {
		int i, x, y;
		Kosarkas k;
		FILE *f = fopen("kosarkasi.txt", "r");
	
		fscanf(f, "%d", &broj_kosarkasa);
		for (i = 0; i < broj_kosarkasa; i++){
			fscanf(f, "%d %d", &x, &y);
			k = Kosarkas(x, y);
			kosarkasi.push_back(k);
		}
		fclose(f);
	}
	
	/* rasporedi podatke */
	poKoliko = broj_kosarkasa / world.size();	
	broadcast(world, poKoliko, 0);
	mojiKosarkasi = new Kosarkas[poKoliko];
	scatter(world, kosarkasi, mojiKosarkasi, poKoliko, 0);
	
	/*obrada */
	//1. uctivanje statistike za moje igrace
	int a, b, c, d;
	FILE *f = fopen("statistika.txt", "r");
	while (fscanf(f, "%d %d %d %d", &a, &b, &c, &d) == 4){
		//2. prodji po svim (mojim) kosarkasima
		for (i = 0; i < poKoliko; i++){
			if (mojiKosarkasi[i].IsPlayer(a)){ //ako se statistika odnosi na trenutnog
				mojiKosarkasi[i].AddStat(b, c, d); //dodaj mu podatke iz fajla
				mojiKosarkasi[i].br_utakmica++;
			}
		}
	}
	fclose(f);
	
	//3. svaki proc odredi svog najboljeg igraca
	
	Kosarkas najKosarkasi_root[world.size()];
	
	Kosarkas mojNaj = mojiKosarkasi[0];
	
	for (i = 1; i < poKoliko; i++)
		if (mojiKosarkasi[i] > mojNaj){
			mojNaj = mojiKosarkasi[i];
		}
	
	/* skupljanje podataka */
	//svi dodaju svog najboljeg na root-ov niz najboljih kosarkasa
	gather(world, mojNaj, 1, najKosarkasi_root, 0);
	
	
	/* stampanje rezultata */
	if (world.rank == 0){
		
		Kosarkas najRoot = najKosarkasi_root[0];
		
		for (i = 1; i < world.size(); i++){
			if (najKosarkasi_root[i] > najRoot)
				najRoot = najKosarkasi_root[i];
		}
		
		printf("MVP je: %d \n", najRoot->ime);
	}
	
	return 0;
}

















