#include <stdio.h>
#include <stdlib.h>
#include <mpi.h>

double P(double x1, double y1, double x2, double y2,
		 double x3, double y3, double x4, double y4) {
	return ((y1 * (x4 - x2) + y2 * (x1 - x3) + y3 * (x2 - x4) + y4 * (x3 - x1)) * 0.5);	
}

main(int argc, char **argv) {
	
	int myid, np, i, j;
	int br_tacaka;
	
	MPI_Init(&argc, &argv);
	MPI_Comm_size(MPI_COMM_WORLD, &np);
	MPI_Comm_rank(MPI_COMM_WORLD, &myid);
	MPI_Status status;
	
	double *X, *Y;
	double p_max = -1.0;
	double max_koord[8];

	if (myid == 0) {
		
		FILE *f = fopen("ulaz.dat", "r");
		
		fscanf(f, "%d", &br_tacaka);
		
		X = (double *)malloc(br_tacaka * sizeof(double));
		Y = (double *)malloc(br_tacaka * sizeof(double));		
		
		for (i = 0; i < br_tacaka; i++) fscanf(f, "%lf%lf", &X[i], &Y[i]);
		
		//for (i = 0; i < br_tacaka; i++) printf("%6.2lf %6.2lf\n", X[i], Y[i]);
	}
	
	// saljem svima ukupan broj tacaka
	MPI_Bcast(&br_tacaka, 1, MPI_INT, 0, MPI_COMM_WORLD);
	
	// svi osim root-a rade alokaciju, jer je root vec uradio
	if (myid != 0) {
		X = (double *)malloc(br_tacaka * sizeof(double));
		Y = (double *)malloc(br_tacaka * sizeof(double));	
	}
	
	// saljem svima nizove tacaka
	MPI_Bcast(X, br_tacaka, MPI_DOUBLE, 0, MPI_COMM_WORLD);
	MPI_Bcast(Y, br_tacaka, MPI_DOUBLE, 0, MPI_COMM_WORLD);
		
	double max_p = -1.0;
	double temp_p = 0.0;
	double koord[8];
	
	// racunam najvecu povrsinu i cuvam koordinate najveceg cetvorougla
	for (i = myid; i < br_tacaka - 3; i += np) {
		for (j = i + 1; j < br_tacaka - 2; j++) {
			temp_p = P(X[i], Y[i], X[j], Y[j], X[j+1], Y[j+1], X[j+2], Y[j+2]);
			if (max_p < temp_p) {
				max_p = temp_p;
				koord[0] = X[i]; koord[1] = Y[i];
				koord[2] = X[j]; koord[3] = Y[j];
				koord[4] = X[j+1]; koord[5] = Y[j+1];
				koord[6] = X[j+2]; koord[7] = Y[j+2];
			}
		}
	}
	
	// radim send - receive da bih mogao da znam od kog procesa sam dobio max temp
	
	if (myid == 0) {
		double p_temp = 0.0;
		double koord_temp[8];
		p_max = max_p;
		for (j = 0; j < 8; j++) {
			max_koord[j] = koord[j];
		}
		for (i = 1; i < np; i++) {
			MPI_Recv(&p_temp, 1, MPI_DOUBLE, i, 20, MPI_COMM_WORLD, &status);
			MPI_Recv(koord_temp, 8, MPI_DOUBLE, i, 21, MPI_COMM_WORLD, &status);
			if (p_temp > p_max) {
				p_max = p_temp;
				for (j = 0; j < 8; j++) {
					max_koord[j] = koord_temp[j];
				}
			}
		}
	}
	else {
		MPI_Send(&max_p, 1, MPI_DOUBLE, 0, 20, MPI_COMM_WORLD);
		MPI_Send(koord, 8, MPI_DOUBLE, 0, 21, MPI_COMM_WORLD);
	}
	
	// root proces stampa povrsinu u koordinate najveceg cetvorougla
	
	if (myid == 0) {
		printf("Povrsina najveceg cetvorougla: %lf\n", p_max);
		printf("Koordinate tog cetvorougla su:\n");
		for (i = 0; i < 8; i += 2) printf("%6.2lf %6.2lf\n", max_koord[i], max_koord[i+1]);
	}
			
	MPI_Finalize();
	return 0;	
}
